#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <unistd.h>
#include <sys/types.h>		// open()
#include <unistd.h>		// read() / write()
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#include <stdio.h>		// FILE
#include <stdlib.h>		// atol()
#include <string.h>		// strstr()
#include <errno.h>          	// EIO, E...


#include <linux/adwin.h>

#include "libadwin.h"
#include "transputer.h"
#include "libadwin/errno.h"

extern long g_Version;

#define g_QNAN_MASKE 0x7F800000

#define FLT_MAX 3.402823466e+38F

#define BOOL    int
#define TRUE    1
#define FALSE   0

#define DUMP(x)	printf("%s = %d\n", #x, x)


/*
//-----------------------------------------------------------------------------
int T_CreateData(adwin_dev_t *dev, int fd, int datanr, int type, long groesse, int ort, int art) {
	errno = ENOSYS;
	return -1;
}
*/
//-----------------------------------------------------------------------------
int T_Get_Memory_dsp(adwin_dev_t *dev, int fd, long base_addr, long anzahl, long *dest) {
	adwin_errno = -ENOTSUP;
	return -1;
}

//-----------------------------------------------------------------------------


int adlink_Testeingang(int dev) {
    long tmp = 0;
    while((ioctl(dev, ADIOGREADSTATUS)) && (tmp++ < 50000));

    return (tmp == 50000) ? 1 : 0;
}

int adlink_Testausgang(int dev) {
    long tmp = 0;
    while((ioctl(dev, ADIOGWRITESTATUS)) && (tmp++ < 50000));

    return (tmp == 50000) ? 1 : 0;
}

//-----------------------------------------------------------------------------
// Hilfsmakros - schliessen der Gertedatei im Fehlerfall
// 	d	int, Nr. des Filezeigers
//	ptr	(char-)Zeiger, woher gelsen/wohin geschrieben werden soll
//	n	wieviele Bytes
//	err	Fehlerrckgabe der Funktion
//
#define write_save(d, ptr, n)		\
if(write((d), (ptr), (n)) < (n)) {		\
	adwin_errno = k_timeout_to_link;		\
	errno = ETIMEDOUT;		\
	return -1;				\
}

#define read_save(d, ptr, n)		\
if(read((d), (ptr), (n)) < (n)) {		\
	adwin_errno = k_timeout_from_link;	\
	errno = ETIMEDOUT;		\
	return -1;				\
}

#define testausgang_save(d)		\
if(adlink_Testausgang((d))) {			\
	adwin_errno = k_timeout_to_link;		\
	errno = ETIMEDOUT;		\
	return -1;				\
}

#define testeingang_save(d)		\
if(adlink_Testeingang(d)) {			\
	adwin_errno = k_timeout_from_link;	\
	errno = ETIMEDOUT;		\
	return -1;				\
}

#define read_save_and_free(fd, dest, cnt, freeme)		\
if(read((fd), (dest), (cnt)) < (cnt)) {		\
	adwin_errno = k_timeout_from_link;	\
	errno = ETIMEDOUT;			\
	free(freeme);					\
	return -1;						\
}

#define write_save_and_free(fd, src, cnt, freeme)		\
if(write((fd), (src), (cnt)) < (cnt)) {		\
	adwin_errno = k_timeout_to_link;	\
	errno = ETIMEDOUT;			\
	free(freeme);					\
	return -1;						\
}


//-----------------------------------------------------------------------------
//
//	Code fr T450 entfernt, da ausgestorben
//
long T_Boot(adwin_dev_t *dev, int fd, const char *dateiname, long iboardsize) {
	FILE *btl;

	/* Returnwert <= 1000 => Fehler */
	unsigned char wert;
	char dat[70] = "\x3F\x00\x00\x3C\x00I:\\ICTOOLS\\ITOOLS\\ISERVER.EXE /sb d:transp\\t2neu\\adwin2.btl     ";
	long wert2;
	long anzahl = 0;
	long groesse_treiber;

	long pos = 0;

	unsigned char senden[100000];

	long return_wert = iboardsize;

	errno = 0;
;
	ioctl(fd, ADIORESETLINK);
	ioctl(fd, ADIOSKDEVPROC, DP_T8);

	if ((btl = fopen(dateiname, "rb" )) == NULL) {
		errno = ENOENT;
		adwin_errno = -errno;
    		return -1;		/* errno bereits gesetzt */
	}

	fseek(btl, pos, SEEK_SET);
	groesse_treiber = fread(senden + anzahl, 1 ,100000, btl);
   	fclose(btl);

	write_save(fd, senden, groesse_treiber);
	testeingang_save(fd);
	read_save(fd, &wert, 1);

	dev->dev_processor = (wert == 8) ? DP_T2 : DP_T8;

	testeingang_save(fd);
	read_save(fd, &wert2, 1);
	read_save(fd, senden, wert);

	if (dev->dev_processor != DP_T2) {			// eigentlich == 2
        	// Fr T800
		if (iboardsize >= 1000000) {
			senden[0] = 0x0B;
			senden[1] = 0;
			senden[2] = 0;
			senden[3] = 8;
			senden[4] = 0;
			senden[5] = 0x23;
			sprintf((char *)(senden + 6) , "%ld" , iboardsize);

			write_save(fd, senden, 13);
		} else {
			senden[0] = 0x0A;
			senden[1] = 0;
			senden[2] = 0;
			senden[3] = 7;
			senden[4] = 0;
			senden[5] = 0x23;
			sprintf((char *)(senden + 6) , "%ld" , iboardsize);

			write_save(fd, senden, 12);
		}

		read_save(fd, senden, 10);
   	}

	senden[0] = 5;
	senden[1] = 0;
	senden[2] = 0;
	senden[3] = 0x0E;
	senden[4] = 1;
	senden[5] = 1;
	senden[6] = 1;

	write_save(fd, senden, 7);
	read_save(fd, senden, 10);
	memcpy(senden, dat, 0x41);	// die wilde Zeichenfolge am Anfang
	write_save(fd, senden, 0x41);

	read_save(fd, senden, 10);
  	senden[0] = 3;
	senden[1] = 0;
	senden[2] = 0;
	senden[3] = 0;
	senden[4] = 0;
	write_save(fd, senden, 5);

	if (dev->dev_processor == DP_T8) {
		read_save(fd, senden, 10);

		senden[0] = 3;
		senden[1] = 0;
		senden[2] = 0;
		senden[3] = 1;
		senden[4] = 0;
		write_save(fd, senden, 5);
	}

	testeingang_save(fd);

	read_save(fd, &wert, 1);
	read_save(fd, senden, wert + 1);

	senden[0] = 3;
	senden[1] = 0;
	senden[2] = 0;
	senden[3] = 0x22;
	senden[4] = 0;
	write_save(fd, senden, 5);

   	if (dev->dev_processor == DP_T2) {
		read_save(fd, senden, 8);
		testausgang_save(fd);

		wert = 0;
		write_save(fd, &wert, 1);
	} else {
		testeingang_save(fd);

		read_save(fd, &wert, 1);
		read_save(fd, senden, wert + 1);

		senden[0] = 3;
		senden[1] = 0;
		senden[2] = 0;
		senden[3] = 0x15;
		senden[4] = 0;
		write_save(fd, senden, 5);

		// Hier nderung
		// entweder falsche Karte oder getenv
		testeingang_save(fd);

		read_save(fd, &wert2, 1);
		if (wert2 == 14) {
			// hier neu
			//******************************************
			//Iboardsize
			read_save(fd, senden, 15);

			senden[0] = 0xA;
			senden[1] = 0;
			senden[2] = 0;
			senden[3] = 7;
			senden[4] = 0;
			senden[5] = 0x23;
			sprintf((char *)(senden + 6) , "%i" , (int)iboardsize);
			write_save(fd, senden, 12);

			//******************************************
			// wieder alt oder Ausgabe das Iboardsize nicht stimmt
			testeingang_save(fd);

			read_save(fd, &wert, 1);
			if (wert != 8) {
				// Fehler mit IBOARDSIZE
				read_save(fd, senden, wert + 1);
				senden[0] = 3;
				senden[1] = 0;
				senden[2] = 0;
				senden[3] = wert - 7;
				senden[4] = 0;
				write_save(fd, senden, 5);

				testeingang_save(fd);

				read_save(fd, &wert, 1);
				read_save(fd, senden, wert + 1);

				senden[0] = 3;
				senden[1] = 0;
				senden[2] = 0;
				senden[3] = 28;
				senden[4] = 0;
				write_save(fd, senden, 5);

				testeingang_save(fd);

				read_save(fd, &wert, 1);
				read_save(fd, senden, wert + 1);
				
				switch (senden[38]) {
					case 0x31:	return_wert = (senden[39] == 0x36) ? 1000000 : 100000;	break;
					case 0x32:	return_wert =  200000;	break;
					case 0x33:	return_wert = 2000000;	break;
					case 0x34:	return_wert =  400000;	break;
					case 0x35:	return_wert =   80000;	break;
					case 0x38:	return_wert =  800000;	break;
				}

				senden[0] = 3;
				senden[1] = 0;
				senden[2] = 0;
				senden[3] = wert - 7;
				senden[4] = 0;
				write_save(fd, senden, 5);
				testeingang_save(fd);
				read_save(fd, &wert2, 1);
			}
		}

		read_save(fd, senden, 9);

		senden[0] = 1;
		senden[1] = 0;
		senden[2] = 0;
		write_save(fd, senden, 3);
	}

	errno = 0;
	return return_wert;
}
//-----------------------------------------------------------------------------

processor_type_t T_Processor_Type(adwin_dev_t *dev, int fd, processor_type_t *processor) {
        // not yet implemented
        *processor = -1;
	return -1;
}

//-----------------------------------------------------------------------------
static long T_AD_Memory (adwin_dev_t *dev, int fd) {
	long mem;
	unsigned char tmp = 253;

	write_save(fd, &tmp, 1);
	read_save(fd, &mem, 4);

	return (mem) ;
}

//-----------------------------------------------------------------------------
int T_Free_Mem(adwin_dev_t *dev, int fd, int Mem_Spec, int *dest) {
	*dest = T_AD_Memory(dev, fd);
	return (*dest == -1 ? -1 : 0);
}

//-----------------------------------------------------------------------------
int T_Test_Version(adwin_dev_t *dev, int fd) {
	char buf[4];
        unsigned char tmp = 255;
	int btl;

	write_save(fd, &tmp, 1);	// 3 == Transputer meldet sich nicht
	read_save(fd, &tmp, 1);

        switch(tmp) {
		case 118:
			/* Przessor */
			read_save(fd, &tmp, 1);
			switch (tmp) {
				case 2:	  dev->dev_processor = DP_T2; break;
				case 4:	  dev->dev_processor = DP_T4; break;
				case 8:	  dev->dev_processor = DP_T8; break;
				default:  dev->dev_processor = DP_UNKNOWN; break;
			}
			ioctl(fd, ADIOSKDEVPROC, dev->dev_processor);

			/* Kartentyp, immer 1 */
			read_save(fd, &tmp, 1);
			/* dev->dev_card_version =(long) tmp; */

			/* BTL-Version */
			read_save(fd, buf, 4);
			btl = *((long *)buf);

			return ((btl/1000) != (g_Version/1000) ? 1 : 0);

	    	case 119:	    	// alter Treiber
		        return 1;

		default:        	// 255 - Transputer meldet sich nicht
			return 3;
	}
}

//-----------------------------------------------------------------------------

#define EXTRACT_FROM_BINFILE(binfile_ptr)			\
({								\
	long __ret=0;						\
	char buf[100];						\
	char *ptr = buf;					\
								\
	for( ; *(binfile_ptr) != 0x46; (binfile_ptr)++, ptr++)	\
		*ptr = *(binfile_ptr);				\
								\
	*ptr = '\0';						\
	(binfile_ptr)++;					\
	__ret = atol(buf);					\
	__ret;							\
})



// short T_ADBload(char *dateiname, adwin_dev_t *dev) {
int T_Load_Process(adwin_dev_t *dev, int fd, const char *dateiname) {
	FILE *f;
	long Basicversion;

	long anzahl,anzahl2,groesse,stacklaenge;
	long anzahlpatches;
	int lauf;
	unsigned char anzahlglvar;
	char art,ProcID;
	short index;
	char binfile[10000], *pBinfile;

	unsigned int *ui_senden;
	long *dw_senden;
	char erg[100];

	long Version = 0;
	int neu;

	int fehlermeldung;		// Rckgabewert
	long delay = 0, anzahlloop = 0;

	char senden[1000000];


	anzahl = 0 ;
	neu = 0;

        if (T_Test_Version(dev, fd) > 1) {
        	 adwin_errno = k_wrong_version;
        	 return 0;
        }

	if ((f = fopen(dateiname, "rb" )) == NULL) {
		adwin_errno = k_File_not_found;
		return 0;
	}

	/* Versionskontrolle  Basicfile */
	fseek(f, -1, SEEK_END);
        fread(&art, 1, 1, f);

	/* Es steht noch keine Compilerversion in der Binrdatei */
	if ((art == 0) || (art == 70)) {
		neu = 1;
	} else {
		fseek(f, -7, SEEK_END);
		fread(erg, 7, 1, f);
		erg[7] = '\0';
		Version = atol(erg);
	}

	/* WARUM DAS DENN ? */
	fclose(f);
	f = fopen(dateiname, "rb" );

	fseek(f, 0, SEEK_SET);
	fread(binfile, 1, 9800, f);

	pBinfile = binfile;
	if (neu == 0){
		Basicversion = EXTRACT_FROM_BINFILE(pBinfile);
	}
	
	ProcID = *pBinfile++;

	senden[0] = 200+ ((ProcID -1) * 2);

/*	if ((ProcID == 2) && (dev->dev_processor == DP_T450)) {
		senden[1] = (*pBinfile++ == 1) ? 1 : 0;

		anzahl = EXTRACT_FROM_BINFILE(pBinfile);
		stacklaenge = EXTRACT_FROM_BINFILE(pBinfile);

		memcpy(senden + 2, &stacklaenge, sizeof(stacklaenge));
		senden[6] = anzahlglvar = *pBinfile++;

		write_save(fd, senden, 7);
	} else {
*/
		anzahl = EXTRACT_FROM_BINFILE(pBinfile);
		stacklaenge = EXTRACT_FROM_BINFILE(pBinfile);

		memcpy(senden + 1, &stacklaenge, sizeof(stacklaenge));
		senden[5] = anzahlglvar = *pBinfile++;;
		write_save(fd, senden, 6);
/*	} */

        for (lauf = 1; lauf <= anzahlglvar; lauf++) {
//		pBinfile++;
                index = EXTRACT_FROM_BINFILE(pBinfile);
		art = *pBinfile++;
		pBinfile ++;
		groesse = EXTRACT_FROM_BINFILE(pBinfile);

		ui_senden = (unsigned int *)(senden);
		ui_senden[0] = index;
		senden[2]    = art;

		dw_senden = (long *)(senden + 3);
		dw_senden[0] = groesse;

		write_save(fd, senden, 7);
	}

	anzahl2 = (anzahl % 2) ? anzahl + 1 : anzahl;

	memcpy(senden, &anzahl2, 4);
	fseek(f, pBinfile - binfile - 1, SEEK_SET);
	fread(senden + 4, 1, anzahl, f);
	write_save(fd, senden, anzahl2 + 4);

	/* Patches */
	fread(binfile, 1, 9800, f);
	fclose(f);
	pBinfile = binfile;

	// Variante "x = (cond) ? foo : bar" wegen Makro nicht mglich
	if (Version / 1000 >= 2000)
		anzahlpatches = EXTRACT_FROM_BINFILE(pBinfile);
	else
		anzahlpatches = *pBinfile++;

	write_save(fd, &anzahlpatches, 4);

   	for (lauf = 1; lauf <= anzahlpatches; lauf++) {
		anzahl  = EXTRACT_FROM_BINFILE(pBinfile);
		groesse = EXTRACT_FROM_BINFILE(pBinfile);

		memcpy(senden, &anzahl, 4);
		memcpy(senden + 4, &groesse, 4);

		write_save(fd, senden, 8);
	}

	if (Version /1000 >= 1004) {
        	delay      = EXTRACT_FROM_BINFILE(pBinfile);
	       	anzahlloop = EXTRACT_FROM_BINFILE(pBinfile);
	}

	if (Version / 1000 >= 2000) {
		/* informtionen ber event,prio,finish pointer auslesen und ausgeben */
                senden[0] = *pBinfile;				/* Prio */
                EXTRACT_FROM_BINFILE(pBinfile);			/* weiter */

                senden[1] = *pBinfile;				/* Event */
                EXTRACT_FROM_BINFILE(pBinfile);			/* weiter */

                groesse = EXTRACT_FROM_BINFILE(pBinfile);	/* Pointer_to_finish */

                memcpy(senden + 2, &groesse, sizeof(groesse));
		write_save(fd, senden, 6);
	} else {
		memset(senden, 0, 6);
		senden[1] = -1;
		write_save(fd, senden, 6);
	}

	fehlermeldung = 1;
/*	if (dev->dev_processor == DP_T450)  (T450)
		read_save(fd, &fehlermeldung, 1);
	}
*/
	if (Version / 1000 >= 1004) {
		// Delay und Anzahl Loops setzen
		T_Set_Par(dev, fd, 910 + ProcID, delay);
		T_Set_Par(dev, fd, 920 + ProcID, anzahlloop);
	}

	return fehlermeldung;
}

//-----------------------------------------------------------------------------
 int T_Start_Process (adwin_dev_t *dev, int fd, int processID) {
	unsigned char tmp[2] = { 32, (unsigned char)processID };
	write_save(fd, tmp, 2);
        return 1;
}
//-----------------------------------------------------------------------------
 int T_Stop_Process (adwin_dev_t *dev, int fd, int processID) {
	unsigned char tmp[2] = { 33, (unsigned char)processID };
	write_save(fd, tmp, 2);
        return 0;
}

//-----------------------------------------------------------------------------
int T_Get_Par (adwin_dev_t *dev, int fd, int par, int *dest) {
	char buf[3];

	buf[0] = 38;
	memcpy(buf + 1, &par, 2);

	write_save(fd, buf, 3);
	read_save(fd, dest, 4);

	return 0;
}

//-----------------------------------------------------------------------------
int T_Get_FPar (adwin_dev_t *dev, int fd, int par, float *dest) {
	char buf[3];
	if (dev->dev_processor == DP_T8) {
		buf[0] = 38;
		memcpy(buf + 1, &par, 2);

		write_save(fd, buf, 3);
		read_save(fd, dest, 4);

		return 0;
	}
	else
		errno = ENOTSUP;
	return -1;
}

//-----------------------------------------------------------------------------
int T_Set_Par (adwin_dev_t *dev, int fd, int par, int32_t val) {
	char buf[7];

	buf[0] = 34;
	memcpy(buf + 1, &par, 2);
	memcpy(buf + 3, &val, 4);

	write_save(fd, buf, 7);

	return 0;
}

//-----------------------------------------------------------------------------
int T_Set_FPar (adwin_dev_t *dev, int fd, int par, float val) {
	char buf[7];
	if (dev->dev_processor == DP_T8) {
		buf[0] = 34;
		memcpy(buf + 1, &par, 2);
		memcpy(buf + 3, &val, 4);

		write_save(fd, buf, 7);
		return(0);
	}
	else
		errno = ENOTSUP;
	return -1;
}

//-----------------------------------------------------------------------------
int T_Workload (adwin_dev_t *dev, int fd, int priority, int32_t *dest) {
	char cmd = 254;
        short val;

	write_save(fd, &cmd, 1);
	read_save(fd, &val, (signed)sizeof(val));

	return val;
}

//-----------------------------------------------------------------------------
int T_Data_Length(adwin_dev_t *dev, int fd, long DataNo) {
	unsigned char cmd[2] = { 116, (unsigned char)DataNo };
	int val;

	write_save(fd, &cmd, 2);
	read_save(fd, &val, 4);

	/* val == -1 wenn data nicht angelegt! */
	return val;
}

//-----------------------------------------------------------------------------
int T_Clear_Data(adwin_dev_t *dev, int fd, int Datanr) {
	char cmd[2] = { 101, (unsigned char)Datanr };

	write_save(fd, &cmd, 2);

	return 1;
}

//-----------------------------------------------------------------------------
int T_Fifo_Clear(adwin_dev_t *dev, int fd, int fifono) {
	char cmd[2] = { 114, (unsigned char)fifono };

	write_save(fd, &cmd, 2);

	return 1;
}

//-----------------------------------------------------------------------------
int T_GetData(adwin_dev_t *dev, int fd, int dataNo, long startindex, long count, void *dest) {
	int data_type = 0;
	char cmd[10];

	cmd[0] = 107;
	cmd[1] = (unsigned char) dataNo;
	memcpy(cmd + 2, &startindex, 4);
	memcpy(cmd + 6, &count, 4);

	write_save(fd, cmd, 10);
	read_save(fd, &data_type, 1);

	if (data_type == 0) {
		errno = EINVAL;
		adwin_errno = k_Data_is_to_small;
		return -1;
	}

	read_save(fd, dest, count * (data_type == 2 ? 2 : 4));

	return 0;
}


//-----------------------------------------------------------------------------
int T_SetData(adwin_dev_t *dev, int fd, int dataNo, long startindex, long count, void *src) {
	int data_type;
	char cmd[10];

	cmd[0] = 115;
	cmd[1] = (unsigned char) dataNo;
	memcpy(cmd + 2, &startindex, 4);
	memcpy(cmd + 6, &count, 4);

        write_save(fd, cmd, 10);
        read_save(fd, &data_type, 1);

	if (data_type == 0) {
		adwin_errno = k_Data_is_to_small;
		errno = EINVAL;
		return -1;
	}

	write_save(fd, src, count * (data_type == 2 ? 2: 4));
	
	return 0;
}

//-----------------------------------------------------------------------------
int T_Fifo_Full(adwin_dev_t *dev, int fd, int fifono, int *dest) {
	char buf[2] = { 112, (char)fifono };

	write_save(fd, buf, 2);
	read_save(fd, dest, 4);

	return 0;
}

//-----------------------------------------------------------------------------
int T_Fifo_Empty(adwin_dev_t *dev, int fd, int fifono, int *dest) {
	char buf[2] = { 113, (char)fifono };

	write_save(fd, buf, 2);
	read_save(fd, dest, 4);

	return 0;
}

//-----------------------------------------------------------------------------
int T_GetFifo(adwin_dev_t *dev, int fd, int fifono, long count, void *dest) {
	int typ, fifo_type = 0;
	char cmd[10];

	cmd[0] = 110;
	cmd[1] = (unsigned char) fifono;
	memcpy(cmd + 2, &count, 4);

	/* This works. A fifo can still be accessed as data ... */
	typ = T_GetDataType(dev, fd, fifono);

	write_save(fd, cmd, 6);
	read_save(fd, &fifo_type, 1);

	if (fifo_type == 0) {
		errno = EINVAL;
		adwin_errno = k_Fifo_to_less_data;
		return -1;
	}

	read_save(fd, dest, count * ((typ == 1) ? 2 : 4));
	return 0;
}

//-----------------------------------------------------------------------------
int T_SetFifo(adwin_dev_t *dev, int fd, int fifono, long count, void *src) {
	int typ, fifo_type;
	char cmd[10];

	cmd[0] = 111;
	cmd[1] = (unsigned char) fifono;
	memcpy(cmd + 2, &count, 4);

	/* This works. A fifo can still be accessed as data ... */
	typ = T_GetDataType(dev, fd, fifono);

        write_save(fd, cmd, 6);
        read_save(fd, &fifo_type, 1);

	if (fifo_type == 0) {
		adwin_errno = k_Fifo_is_to_small;
		errno = EINVAL;
		return -1;
	}

	write_save(fd, src, count * ((typ == 1) ? 2 : 4));
	return 0;
}

//-----------------------------------------------------------------------------
int T_GetDataType(adwin_dev_t *dev, int fd, int datanr) {
	unsigned char type, buf[10] = { 107, (unsigned char)datanr };
	int * ptr = (int *)(buf + 2);
	int tmp;

	ptr[0] = ptr[1] = 1;	/* first, count */

	write_save(fd, buf, 10);
	read_save(fd, &type, 1);
	
	read_save(fd, &tmp, (type == 1) ? 2 /* 16bit */ : 4 /* 32bit */);
	return type;
}

//-----------------------------------------------------------------------------
int  T_Get_Par_All(adwin_dev_t *dev, int fd, int first, int count, int32_t *dest) {
	unsigned char buf[3] = { 117, (unsigned char)first, (unsigned char)count };

	write_save(fd, buf, 3);
	read_save(fd, dest, count * 4);

	return 0;
}

//-----------------------------------------------------------------------------
int  T_Get_FPar_All(adwin_dev_t *dev, int fd, int first, int count, float *dest) {
	unsigned char buf[3] = { 117, (unsigned char)(first), (unsigned char)count };

	write_save(fd, buf, 3);
	read_save(fd, dest, count * 4);

	return 0;
}

//-----------------------------------------------------------------------------
int  T_Clear_Process(adwin_dev_t *dev, int fd, int proc_nr) {
	errno = ENOSYS;
	return -1;
}
